package org.processmining.models.graphbased.directed.socialnetwork;

import java.awt.Color;
import java.awt.Dimension;

import org.processmining.framework.util.Cast;
import org.processmining.framework.util.HTMLToString;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphNode;
import org.processmining.models.shapes.Ellipse;

public class SNNode extends AbstractDirectedGraphNode {

	/**
	 * this object identifies the node.
	 */
	private final Object identifier;
	private final SocialNetworkImpl graph;

	public SNNode(Object identifier, SocialNetworkImpl graph) {
		super();
		this.identifier = identifier;
		this.graph = graph;
		getAttributeMap().put(AttributeMap.SHAPE, new Ellipse());
		getAttributeMap().put(AttributeMap.SQUAREBB, false);
		getAttributeMap().put(AttributeMap.RESIZABLE, true);
		getAttributeMap().put(AttributeMap.SIZE, new Dimension(40, 30));
		getAttributeMap().put(AttributeMap.FILLCOLOR, new Color(90, 90, 90));
		if (identifier instanceof HTMLToString) {
			getAttributeMap().put(AttributeMap.LABEL, Cast.<HTMLToString>cast(identifier).toHTMLString(true));
		} else {
			getAttributeMap().put(AttributeMap.LABEL, identifier.toString());
		}
		getAttributeMap().put(AttributeMap.SIZE, new Dimension(50, 50));
	}

	public Object getIdentifier() {
		return identifier;
	}

	public boolean equals(Object o) {
		return (o instanceof SNNode ? identifier.equals(((SNNode) o).identifier) : false);
	}

	public int hashCode() {
		return identifier.hashCode();
	}

	@Override
	public SocialNetworkImpl getGraph() {
		return graph;
	}

	public int getInDegree() {
		return graph.getInDegree(this);
	}

	public int getOutDegree() {
		return graph.getOutDegree(this);
	}

	public int getDegree() {
		return graph.getDegree(this);
	}

	public double getInWeightDegree() {
		return graph.getInWeightDegree(this);
	}

	public double getOutWeightDegree() {
		return graph.getOutWeightDegree(this);
	}

	public double getWeightDegree() {
		return graph.getWeightDegree(this);
	}

}
