package org.processmining.socialnetwork.models;

import java.util.Collection;

import org.processmining.socialnetwork.models.impl.TObjectDoubleHashMapSocialNetworkImpl;

/**
 * A social network is a directed graph having resources as vertices.
 * 
 * @author S.J. van Zelst
 *
 * @param <T>
 *            type of vertices in network
 */
public interface SocialNetwork<T> extends Cloneable {

	enum Implementation {
		TROVE_DOUBLE_HASH_MAP;
	}

	/**
	 * adds an arc (and possibly vertices if not present) to the network
	 * 
	 * @param from
	 * @param to
	 * @param value
	 */
	void addArc(T from, T to, double value);

	void addVertex(T vertex);

	void clear();

	boolean containsArc(T from, T to);

	double getArcValue(T from, T to);

	Collection<T> getInSet(T to);

	Collection<T> getOutSet(T from);

	Collection<T> getVertices();

	void removeArc(T from, T to);

	SocialNetwork<T> clone();

	Implementation getImplementation();

	public class Factory {

		public static <T> SocialNetwork<T> createSocialNetwork(SocialNetwork.Implementation implementation) {
			switch (implementation) {
				case TROVE_DOUBLE_HASH_MAP :
				default :
					return new TObjectDoubleHashMapSocialNetworkImpl<>();
			}
		}

	}

}
