package org.processmining.socialnetwork.models;

import java.util.List;

import javax.swing.JComponent;

/**
 * A SocialNetworkMetric is an entity that constructs and maintains a metric (as
 * represented by the internal Metric enum) based on a case trie that is derived
 * from a stream (to be changed to an arbitrary trace view of the stream...)
 * 
 * @author svzelst
 *
 * @param <T>
 *            type of nodes within the network
 */
public interface SocialNetworkMetric<T> {

	//@formatter:off
	enum Metric {
		ABSOLUTE_HANDOVER_OF_WORK("Handover of work (Absolute, Def. 4.4.1)"),
		BOOLEAN_HANDOVER_OF_WORK("Handover of work (Boolean, Def 4.4.2)"), 		
		ABSOLUTE_CAUSAL_HAND_OVER_OF_WORK("Handover of work (Absolute & Causal, Def 4.4.3)"), 
		BOOLEAN_CAUSAL_HAND_OVER_OF_WORK("Handover of work (Boolean & Causal, Def 4.4.4)"),
		ABSOLUTE_INBETWEEN("In-between (Absolute, Def 4.7.1)"),
		BOOLEAN_INBETWEEN("In-between (Boolean, Def 4.7.2)"),
		ABSOLUTE_CAUSAL_INBETWEEN("In-between (Absolute & Causal, Def 4.7.3)"), 
		BOOLEAN_CAUSAL_INBETWEEN("In-between (Boolean & Causal, Def 4.7.4)"),
		WORKING_TOGETHER("Working together (Def 4.8)"),
		JOINT_ACTIVITY_MINKOWSKI_DISTANCE("Joint Activities (Minkowski Distance, Def 4.10.1)"),
		JOINT_ACTIVITY_HAMMING_DISTANCE("Joint Activities (Hamming Distance, Def. 4.10.2"),
		JOINT_ACTIVITY_PEARSON_CORRELATION("Joint Activities (Pearson's Correlation, Def. 4.10.3)");

		private final String name;

		private Metric(final String name) {
			this.name = name;
		}

		@Override
		public String toString() {
			return name;
		}
	}
	//@formatter:on	

	SocialNetwork<T> getNetwork();

	JComponent getParameterEditor(String key);

	List<String> getParameterKeys();

	String getParameterValue(String key);

	List<String> getParameterValues();

	Metric getType();

	void setParameter(String key, String value);

}
