package org.processmining.socialnetwork.models;

import javax.swing.JComponent;

import org.processmining.socialnetwork.models.impl.graphstream.GraphStreamSingleGraphSocialNetworkVisualizationImpl;
import org.processmining.socialnetwork.models.impl.graphviz.GraphVizSocialNetworkVisualizationImpl;

/**
 * A social network visualization is a graph-based visualization of the social
 * network. The implementing underlying library is recorded by the internal
 * Implementation enum. Note that the visualization might be based on some
 * function applied over (a series of) the actual underlying social network(s).
 * Hence, the vertices within the network might refer to groups of resources,
 * rather than individual resources
 * 
 * @author svzelst
 *
 * @param <T>
 *            type of vertices in the network
 */
public interface SocialNetworkVisualization<T> {

	enum Library {
		GRAPH_VIZ("Graph Viz"), GRAPH_STREAM("Graph Stream");

		private final String toStr;

		private Library(String toStr) {
			this.toStr = toStr;
		}

		@Override
		public String toString() {
			return toStr;
		}
	}

	void addArc(T from, T to);

	void addVertex(T vertex);

	boolean containsArc(T from, T to);

	boolean containsVertex(T vertex);

	void clear();

	JComponent get();

	Library getLibrary();

	String getArcLabel(T from, T to);

	void setArcLabel(T from, T to, String label);

	void setArcLabelStyle(T from, T to, SocialNetworkArcLabelStyle style);

	void setArcStyle(T from, T to, SocialNetworkArcStyle style);

	void setArcStyle(T from, T to, SocialNetworkArcStyle... styles);

	void setVertexLabel(T node, String label);

	void setVertexStyle(T node, SocialNetworkVertexStyle style);

	void removeArc(T from, T to);

	void removeVertex(T vertex);

	public class Factory {
		public static <T> SocialNetworkVisualization<T> newInstance(Library implementation) {
			SocialNetworkVisualization<T> visualization = new GraphVizSocialNetworkVisualizationImpl<>();
			switch (implementation) {
				case GRAPH_STREAM :
					visualization = new GraphStreamSingleGraphSocialNetworkVisualizationImpl<>();
					break;
				case GRAPH_VIZ :
					visualization = new GraphVizSocialNetworkVisualizationImpl<>();
					break;
			}
			return visualization;
		}
	}

}
