package org.processmining.socialnetwork.models;

import java.util.List;

import javax.swing.JComponent;

import org.processmining.socialnetwork.models.impl.SimpleSocialNetworkVisualizationAlgorithmImpl;
import org.processmining.socialnetwork.models.impl.WindowedSocialNetworkVisualizationAlgorithmImpl;

/**
 * A SocialNetworkVisualizationAlgorithm is responsible for visualizing the
 * (sequence of) social network(s) generated by the SocialNetworkMetric
 * entities. To do its job, it needs a SocialNetworkVisualization, hence, after
 * creation of an instance of this interface, one has to set the visualization.
 * 
 * @author svzelst
 *
 * @param <T>
 *            node type of the network
 */
public interface SocialNetworkVisualizationAlgorithm<T> {

	enum Algorithm {
		SIMPLE("Simple"), WINDOW("Window");

		private String str;

		private Algorithm(String str) {
			this.str = str;
		}

		@Override
		public String toString() {
			return str;
		}
	}

	void clear();

	List<String> getParameterKeys();

	List<String> getParameterValues();

	SocialNetworkVisualization<T> getVisualization();

	void processNewNetwork(SocialNetwork<T> network);

	void setParameter(String key, String value);

	void setVisualization(SocialNetworkVisualization<T> visualization);

	JComponent visualize();

	Algorithm getAlgorithm();

	public class Factory {

		public static <T> SocialNetworkVisualizationAlgorithm<T> newInstance(Algorithm type) {
			switch (type) {
				case WINDOW :
					return new WindowedSocialNetworkVisualizationAlgorithmImpl<T>();
				case SIMPLE :
				default :
					return new SimpleSocialNetworkVisualizationAlgorithmImpl<>();
			}
		}

		public static <T> SocialNetworkVisualizationAlgorithm<T> newInstance(Algorithm type,
				SocialNetworkVisualization.Library library) {
			SocialNetworkVisualization<T> visualization = SocialNetworkVisualization.Factory.newInstance(library);
			switch (type) {
				case WINDOW :
					return new WindowedSocialNetworkVisualizationAlgorithmImpl<T>(visualization);
				case SIMPLE :
				default :
					return new SimpleSocialNetworkVisualizationAlgorithmImpl<>(visualization);
			}
		}
	}

}
