package org.processmining.socialnetwork.models.impl;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;

import org.processmining.socialnetwork.models.SocialNetworkVisualization;
import org.processmining.socialnetwork.models.SocialNetworkVisualizationAlgorithm;


public abstract class AbstractSocialNetworkVisualizationAlgorithm<T> implements SocialNetworkVisualizationAlgorithm<T> {

	private final List<String> parameterKeys = new ArrayList<>();

	private SocialNetworkVisualization<T> visualization;

	@SuppressWarnings("unchecked")
	public AbstractSocialNetworkVisualizationAlgorithm() {	
		this((SocialNetworkVisualization<T>) SocialNetworkVisualization.Factory
				.newInstance(SocialNetworkVisualization.Library.GRAPH_VIZ));
	}

	public AbstractSocialNetworkVisualizationAlgorithm(SocialNetworkVisualization<T> visualization) {
		this.visualization = visualization;
		setupParameterKeyList(parameterKeys);
	}

	public List<String> getParameterKeys() {
		return parameterKeys;
	}

	public List<String> getParameterValues() {
		return new ArrayList<String>();
	}

	public SocialNetworkVisualization<T> getVisualization() {
		return visualization;
	}

	public void setParameter(String key, String value) {
		//NOP
	}

	protected abstract void setupParameterKeyList(List<String> list);

	public void setVisualization(SocialNetworkVisualization<T> visualization) {
		this.visualization = visualization;
	}

	public JComponent visualize() {
		return visualization.get();
	}
}
