package org.processmining.socialnetwork.models.impl;

import java.text.DecimalFormat;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.processmining.socialnetwork.models.SocialNetwork;
import org.processmining.socialnetwork.models.SocialNetworkVertexStyle;
import org.processmining.socialnetwork.models.SocialNetworkVisualization;
import org.processmining.socialnetwork.models.SocialNetworkVisualizationAlgorithm;

public class SimpleSocialNetworkVisualizationAlgorithmImpl<T> extends AbstractSocialNetworkVisualizationAlgorithm<T> {

	private DecimalFormat decimalFormat = new DecimalFormat("#.##");

	// TODO: add threshold as a parameter!

	private SocialNetwork<T> previousNetwork = null;

	private Object monitor = new Object();

	// candidate for paramter
	private int roundingMode = 3;

	public SimpleSocialNetworkVisualizationAlgorithmImpl() {
		super();
		decimalFormat = constructDecimalFormat(roundingMode);
	}

	public SimpleSocialNetworkVisualizationAlgorithmImpl(SocialNetworkVisualization<T> visualization) {
		super(visualization);
		decimalFormat = constructDecimalFormat(roundingMode);
	}

	private void addVertexToVisualization(T vertex) {
		getVisualization().addVertex(vertex);
		getVisualization().setVertexStyle(vertex, SocialNetworkVertexStyle.HIGH);
		getVisualization().setVertexLabel(vertex, vertex.toString());
	}

	public void clear() {
		synchronized (monitor) {
			previousNetwork = null;
			getVisualization().clear();
		}
	}

	private DecimalFormat constructDecimalFormat(int numZeros) {
		String pattern = numZeros <= 0 ? "#" : "#." + StringUtils.repeat('#', numZeros);
		return new DecimalFormat(pattern);
	}

	private void init(SocialNetwork<T> network) {
		if (network != null) {
			for (T from : network.getVertices()) {
				for (T to : network.getOutSet(from)) {
					addVertexToVisualization(from);
					addVertexToVisualization(to);
					getVisualization().addArc(from, to);
					getVisualization().setArcLabel(from, to, decimalFormat.format(network.getArcValue(from, to)));
				}
			}
		}
	}

	public void processNewNetwork(SocialNetwork<T> network) {
		synchronized (monitor) {
			if (previousNetwork == null) {
				System.out.println("call init(network)");
				init(network);
			} else {
				//CHECK NEW AND EXISTING ARCS
				for (T from : network.getVertices()) {
					for (T to : network.getOutSet(from)) {
						if (!getVisualization().containsArc(from, to)) {
							addVertexToVisualization(from);
							addVertexToVisualization(to);
							getVisualization().addArc(from, to);

						}
						getVisualization().setArcLabel(from, to, decimalFormat.format(network.getArcValue(from, to)));
					}
				}
				//REMOVE OLD ARCS
				for (T from : previousNetwork.getVertices()) {
					for (T to : previousNetwork.getOutSet(from)) {
						if (!network.containsArc(from, to)) {
							getVisualization().removeArc(from, to);
						}
					}
				}
			}
			previousNetwork = network;
		}
	}

	protected void setupParameterKeyList(List<String> list) {
		//NOP
	}

	@Override
	public void setVisualization(SocialNetworkVisualization<T> visualization) {
		super.setVisualization(visualization);
		init(previousNetwork);
	}

	public SocialNetworkVisualizationAlgorithm.Algorithm getAlgorithm() {
		return Algorithm.SIMPLE;
	}

}
