package org.processmining.socialnetwork.models.impl;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import org.processmining.framework.util.Pair;
import org.processmining.socialnetwork.models.SocialNetwork;

import gnu.trove.map.hash.TObjectDoubleHashMap;

public class TObjectDoubleHashMapSocialNetworkImpl<T> implements SocialNetwork<T> {

	private final TObjectDoubleHashMap<Pair<T, T>> network;
	private final Collection<T> vertices;
	private final Map<T, Collection<T>> outArcs;
	private final Map<T, Collection<T>> inArcs;

	public TObjectDoubleHashMapSocialNetworkImpl() {
		this(new TObjectDoubleHashMap<Pair<T, T>>(), new HashSet<T>(), new HashMap<T, Collection<T>>(),
				new HashMap<T, Collection<T>>());
	}

	private TObjectDoubleHashMapSocialNetworkImpl(TObjectDoubleHashMap<Pair<T, T>> network, Collection<T> vertices,
			Map<T, Collection<T>> inArcs, Map<T, Collection<T>> outArcs) {
		this.network = network;
		this.vertices = vertices;
		this.outArcs = outArcs;
		this.inArcs = inArcs;
	}

	public void addArc(T from, T to, double value) {
		addVertex(from);
		addVertex(to);
		network.put(new Pair<T, T>(from, to), value);
		outArcs.get(from).add(to);
		inArcs.get(to).add(from);
	}

	public void addVertex(T vertex) {
		vertices.add(vertex);
		if (!outArcs.containsKey(vertex))
			outArcs.put(vertex, new HashSet<T>());
		if (!inArcs.containsKey(vertex))
			inArcs.put(vertex, new HashSet<T>());
	}

	public void removeArc(T from, T to) {
		if (vertices.contains(from) && outArcs.containsKey(from) && inArcs.containsKey(to)) {
			network.remove(new Pair<T, T>(from, to));
			outArcs.get(from).remove(to);
			if (outArcs.get(from).isEmpty()) {
				outArcs.remove(from);
			}
			checkVertexRemoval(from, inArcs);

			inArcs.get(to).remove(from);
			if (inArcs.get(to).isEmpty()) {
				inArcs.remove(to);
			}
			checkVertexRemoval(to, outArcs);
		}
	}

	private void checkVertexRemoval(T vertex, Map<T, Collection<T>> arcMap) {
		boolean remove = false;
		if (!arcMap.containsKey(vertex)) {
			remove = true;
		} else {
			if (arcMap.get(vertex).isEmpty()) {
				arcMap.remove(vertex);
				remove = true;
			}
		}
		if (remove) {
			vertices.remove(vertex);
		}
	}

	public Collection<T> getVertices() {
		return vertices;
	}

	public Collection<T> getInSet(T to) {
		return !inArcs.containsKey(to) ? new HashSet<T>() : inArcs.get(to);
	}

	public Collection<T> getOutSet(T from) {
		return !outArcs.containsKey(from) ? new HashSet<T>() : outArcs.get(from);
	}

	public void clear() {
		network.clear();
		vertices.clear();
		outArcs.clear();
		inArcs.clear();
	}

	public double getArcValue(T from, T to) {
		return network.get(new Pair<T, T>(from, to));
	}

	public SocialNetwork<T> clone() {
		TObjectDoubleHashMap<Pair<T, T>> nwClone = new TObjectDoubleHashMap<>(network);
		Collection<T> vertClone = new HashSet<T>(vertices);
		Map<T, Collection<T>> outArcsClone = new HashMap<>(outArcs);
		Map<T, Collection<T>> inArcsClone = new HashMap<>(inArcs);
		return new TObjectDoubleHashMapSocialNetworkImpl<>(nwClone, vertClone, inArcsClone, outArcsClone);
	}

	public boolean containsArc(T from, T to) {
		boolean result = false;
		if (outArcs.containsKey(from)) {
			if (outArcs.get(from).contains(to)) {
				result = true;
			}
		}
		return result;
	}

	public org.processmining.socialnetwork.models.SocialNetwork.Implementation getImplementation() {
		return Implementation.TROVE_DOUBLE_HASH_MAP;
	}

}
