package org.processmining.socialnetwork.models.impl.graphviz;

import java.util.HashMap;
import java.util.Map;

import javax.swing.JComponent;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.graphviz.dot.Dot;
import org.processmining.plugins.graphviz.dot.DotEdge;
import org.processmining.plugins.graphviz.dot.DotNode;
import org.processmining.plugins.graphviz.visualisation.DotPanel;
import org.processmining.socialnetwork.models.SocialNetworkArcLabelStyle;
import org.processmining.socialnetwork.models.SocialNetworkArcStyle;
import org.processmining.socialnetwork.models.SocialNetworkVertexStyle;
import org.processmining.socialnetwork.models.SocialNetworkVisualization;

public class GraphVizSocialNetworkVisualizationImpl<T> extends Dot implements SocialNetworkVisualization<T> {

	private final Map<Pair<T, T>, DotEdge> arcs = new HashMap<>();

	private final DotPanel dotPanel = new DotPanel(this);
	private final Map<T, DotNode> vertices = new HashMap<>();

	public GraphVizSocialNetworkVisualizationImpl() {
		setOption("bgcolor", "transparent");
	}

	public void addArc(T from, T to) {
		DotNode fromDot = addOrGetDotNode(from);
		DotNode toDot = addOrGetDotNode(to);
		if (!arcs.containsKey(new Pair<T, T>(from, to))) {
			arcs.put(new Pair<T, T>(from, to), addEdge(fromDot, toDot));
		}
	}

	private DotNode addOrGetDotNode(T node) {
		DotNode nodeInGraph;
		if (!vertices.containsKey(node)) {
			nodeInGraph = addNode(node.toString());
			vertices.put(node, nodeInGraph);
		} else {
			nodeInGraph = vertices.get(node);
		}
		return nodeInGraph;
	}

	public void addVertex(T vertex) {
		addOrGetDotNode(vertex);
	}

	public void clear() {
		for (DotEdge e : arcs.values()) {
			removeEdge(e);
		}
		for (DotNode n : vertices.values()) {
			removeNode(n);
		}
		arcs.clear();
		vertices.clear();
	}

	public boolean containsArc(T from, T to) {
		return arcs.containsKey(new Pair<T, T>(from, to));
	}

	public boolean containsVertex(T vertex) {
		return vertices.containsKey(vertex);
	}

	public JComponent get() {
		dotPanel.changeDot(this, false);
		return dotPanel;
	}
	
	public String getArcLabel(T from, T to) {
		return "";
	}

	public SocialNetworkVisualization.Library getLibrary() {
		return Library.GRAPH_VIZ;
	}

	public void removeArc(T from, T to) {
		Pair<T, T> pair = new Pair<T, T>(from, to);
		removeEdge(arcs.get(pair));
		arcs.remove(pair);
	}

	public void removeVertex(T vertex) {
		removeNode(vertices.get(vertex));
		vertices.remove(vertex);
	}

	public void setArcLabel(T from, T to, String label) {
		DotEdge edge = arcs.get(new Pair<T, T>(from, to));
		edge.setLabel(label);
	}

	public void setArcLabelStyle(T from, T to, SocialNetworkArcLabelStyle style) {
		// TODO Auto-generated method stub

	}

	public void setArcStyle(T from, T to, SocialNetworkArcStyle style) {
		// TODO Auto-generated method stub

	}

	public void setArcStyle(T from, T to, SocialNetworkArcStyle... styles) {
		// TODO Auto-generated method stub

	}

	public void setVertexLabel(T node, String label) {
		// TODO Auto-generated method stub

	}

	public void setVertexStyle(T node, SocialNetworkVertexStyle style) {
		// TODO Auto-generated method stub

	}

}
